import { defineCollection, z } from "astro:content";
import { glob } from "astro/loaders";

const posts = defineCollection({
  loader: glob({ pattern: ["[^_]*.md", "[^_]*/index.mdx"], base: "./src/posts/" }),
  schema: z.object({
    title: z.string(),
    date: z.coerce.date().default(new Date()),
    updated: z.optional(z.coerce.date()),
    tags: z.optional(z.nullable(z.array(z.string()))),

    visibility: z.optional(z.union([z.literal("draft"), z.literal("hidden"), z.literal("visible")]))
  }),
});

const authorLink = z.object({
  name: z.string(),
  href: z.string(),
  class: z.optional(z.literal("mastodon")),
  rel: z.optional(z.literal("me")),
});

const authorPlain = z.object({
  name: z.string(),
  bio: z.optional(z.string()),
  email: z.optional(z.string()),
  default: z.optional(z.boolean()),
  links: z.optional(z.array(authorLink)),
});

const authors = defineCollection({
  loader: glob({ pattern: ["[^_]*.mdx"], base: "./src/authors/" }),
  schema: authorPlain,
});

export const collections = { posts, authors };
