import type { APIRoute } from "astro";
import { getCollection } from "astro:content";
import { postParamlink } from "~/utils/posts";

export const GET: APIRoute = async () => {
  const headers = new Map<string, Headers>();

  headers.set(
    "/*",
    new Headers({
      "Cache-Control": "public, max-age=1440, must-revalidate",
    }),
  );

  headers.set(
    "/_astro/*",
    new Headers({
      "Cache-Control": "public, max-age=31536000, immutable",
    }),
  );

  const posts = await getCollection("posts");

  for (const post of posts) {
    const etagContent = JSON.stringify({
      id: post.id,
      body: post.body,
      data: post.data,
    });
    const etag = `"${new Bun.SHA256().update(etagContent).digest("hex")}"`;

    const matcher = postParamlink(post);
    headers.set(
      matcher,
      new Headers({
        ETag: etag,
      }),
    );

    headers.set(
      `${matcher}index.html`,
      new Headers({
        ETag: etag,
      }),
    );
  }

  const lines = [] as string[];

  for (const [k, entries] of headers) {
    lines.push(k);
    for (const [name, value] of entries) {
      lines.push(`  ${name}: ${value}`);
    }
  }

  return new Response(lines.join("\n"));
};
