import type { APIRoute, GetStaticPaths, Page } from "astro";
import { getCollection, render, type CollectionEntry } from "astro:content";
import { experimental_AstroContainer as AstroContainer } from "astro/container";
import { loadRenderers } from "astro:container";
import { getContainerRenderer as solidContainerRenderer } from "@astrojs/solid-js";
import { getContainerRenderer as mdxContainerRenderer } from "@astrojs/mdx";
import { postParamlink, shouldBeVisible } from "~/utils/posts";

export const getStaticPaths = (async ({ paginate }) => {
  const posts = (await getCollection("posts")).filter(shouldBeVisible).sort(
    (a, b) => b.data.date.getTime() - a.data.date.getTime(),
  );

  return paginate(posts, {pageSize: 5});
}) satisfies GetStaticPaths;

export const GET: APIRoute<{ page: Page<CollectionEntry<"posts">> }> = async ({
  props: { page },
}) => {
  const renderers = await loadRenderers([
    solidContainerRenderer(),
    mdxContainerRenderer(),
  ]);
  const cont = await AstroContainer.create({
    renderers: renderers,
  });
  const authors = await getCollection("authors");

  const hasNextPage = page.currentPage < page.start + 1;

  return new Response(
    JSON.stringify({
      version: "https://jsonfeed.org/version/1.1",
      title: "Rubicon's Rubicon",
      home_page_url: "https://rubicon.lightstands.xyz/",
      feed_url: "https://rubicon.lightstands.xyz/feed.json",
      next_url: hasNextPage
        ? `https://rubicon.lightstands.xyz/feeds/${page.currentPage + 1}.json`
        : undefined,
      authors: authors.map((item) => {
        return {
          name: item.data.name,
          url: item.data.links?.[0].href,
        };
      }),
      items: await Promise.all(
        page.data.map(async (item) => {
          const { Content } = await render(item);
          return {
            id: item.id,
            context_html: await cont.renderToString(Content),
            url: new URL(postParamlink(item), "https://rubicon.lightstands.xyz").toString(),
          };
        }),
      ),
    }, undefined, 2),
  );
};
