import type { CollectionEntry } from "astro:content";

function toPadLeft(n: number) {
  if (n < 10) {
    return `0${n}`;
  } else {
    return n.toString();
  }
}

export function getPostParam(post: CollectionEntry<"posts">) {
  const meta = post.data;
  return {
    year: meta.date.getUTCFullYear(),
    month: toPadLeft(meta.date.getUTCMonth() + 1),
    date: toPadLeft(meta.date.getUTCDate() + 1),
    postid: post.id,
  };
}

export function postParamlink(post: CollectionEntry<"posts">) {
  const meta = post.data;
  return `/${meta.date.getUTCFullYear()}/${toPadLeft(meta.date.getUTCMonth() + 1)}/${toPadLeft(meta.date.getUTCDate() + 1)}/${post.id}/`;
}

export function getAllTags(posts: CollectionEntry<"posts">[]) {
  const tags = new Set(posts.filter(shouldBeVisible).flatMap((post) => post.data.tags || []));

  return tags.values().toArray().sort();
}

function p50Of(histogram: number[]) {
  if (histogram.length === 0) {
    return undefined;
  } else if (histogram.length === 1) {
    return histogram[0];
  }
  const idx = histogram.length / 2;
  if (Math.trunc(idx) !== idx) {
    const leftIdx = Math.trunc(idx);
    const rightIdx = Math.ceil(idx);

    return (histogram[leftIdx] + histogram[rightIdx]) / 2;
  } else {
    return histogram[idx];
  }
}

export function getHotTags(posts: CollectionEntry<"posts">[], tags: string[]) {
  const freqs = new Map(tags.map((x) => [x, 0]));

  for (const post of posts) {
    for (const tag of post.data.tags || []) {
      const count = freqs.get(tag)!;
      freqs.set(tag, count + 1);
    }
  }

  const histogram = freqs.values().toArray().sort();
  const p50 = p50Of(histogram);

  if (p50 === undefined) {
    return [];
  }

  const results = [] as string[];

  for (const [tag, freq] of freqs.entries()) {
    if (freq > p50) {
      results.push(tag);
    }
  }

  return results;
}

export function shouldBeVisible(post: CollectionEntry<"posts">) {
  return !post.data.visibility || post.data.visibility === "visible"
}
