import { match } from "@formatjs/intl-localematcher";

async function selectWikipediaInstance(
  instance: string,
  escapedTitle: string,
  langs: readonly string[],
  userAgent: string,
) {
  const url = `https://${instance}.wikipedia.org/w/rest.php/v1/page/${escapedTitle}/links/language`;

  const response = await fetch(url, {
    headers: {
      "api-user-agent": userAgent,
    },
  });

  if (!(response.ok && response.status === 200)) {
    throw new TypeError("resource not found");
  }

  const languages = (await response.json()) as {
    code: string;
    key: string;
  }[];

  const code = match(
    langs,
    languages.map((x) => x.code),
    instance,
  );

  return languages.find((x) => x.code === code)!;
}

export async function fetchWikipediaSummary(
  instance: string,
  title: string,
  opts: {
    langs?: readonly string[];
    userAgent: string;
  },
) {
  const escapedTitle = encodeURIComponent(title);

  const { key, code } = opts.langs
    ? await selectWikipediaInstance(
        instance,
        escapedTitle,
        opts.langs,
        opts.userAgent,
      )
    : {
        key: title,
        code: instance,
      };

  const baseUrl = `https://${code}.wikipedia.org`;

  const summaryUrl = new URL(
    `${baseUrl}/api/rest_v1/page/summary/${key}?redirect=false`,
  );

  const response = await fetch(summaryUrl, {
    headers: {
      accept:
        'application/json; charset=utf-8; profile="https://www.mediawiki.org/wiki/Specs/Summary/1.4.2"',
      "api-user-agent": `Rubicon's Rubicon (on Server, https://rubicon.lightstands.xyz/external-resource-usage/)`,
    },
  });
  const data = (await response.json()) as {
    extract: string;
    titles: { normalized: string };
    lang: string;
    content_urls: {
      desktop: {
        page: string;
      };
    };
  };

  return data;
}
